MODULE icbutl

   !!======================================================================
   !!                       ***  MODULE  icbutl  ***
   !! Ocean physics:  various iceberg utility routines
   !!======================================================================
   !! History : 3.3.1 !  2010-01  (Martin&Adcroft) Original code
   !!            -    !  2011-03  (Madec)          Part conversion to NEMO form
   !!            -    !                            Removal of mapping from another grid
   !!            -    !  2011-04  (Alderson)       Split into separate modules
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   interp_flds   :
   !!   bilin         :
   !!   bilin_e       :
   !!----------------------------------------------------------------------
   USE par_oce                             ! ocean parameters
   USE dom_oce                             ! ocean domain
   USE in_out_manager                      ! IO parameters
   USE lbclnk                              ! lateral boundary condition
   USE lib_mpp                             ! MPI code and lk_mpp in particular
   USE icb_oce                             ! define iceberg arrays
   USE sbc_oce                             ! ocean surface boundary conditions
#if defined key_lim2
   USE ice_2,         ONLY: u_ice, v_ice   ! LIM-2 ice velocities  (CAUTION in C-grid do not use key_vp option)
   USE ice_2,         ONLY: hi => hicif    ! LIM-2 ice thickness
#elif defined key_lim3
   USE ice,           ONLY: u_ice, v_ice   ! LIM-3 variables  (always in C-grid)
                                           ! gm  LIM3 case the mean ice thickness (i.e. averaged over categories)
                                           ! gm            has to be computed somewhere in the ice and accessed here
#endif

   IMPLICIT NONE
   PRIVATE

   PUBLIC   copy_flds                    ! routine called in icbrun module
   PUBLIC   interp_flds                  ! routine called in icbdyn, icbthm modules
   PUBLIC   bilin                        ! routine called in icbini, icbdyn modules
   PUBLIC   bilin_x                      ! routine called in icbdyn module
   PRIVATE  bilin_e
   PUBLIC   add_new_berg_to_list         ! routine called in icbini.F90, icbclv, icblbc and icbrst modules
   PRIVATE  insert_berg_into_list
   PUBLIC   delete_iceberg_from_list     ! routine called in icblbc, icbthm modules
   PUBLIC   destroy_iceberg              ! routine called in icbrun module
   PUBLIC   track_berg                   ! routine not currently used, retain just in case
   PUBLIC   print_berg                   ! routine called in icbthm module
   PUBLIC   print_bergs                  ! routine called in icbini, icbrun module
   PUBLIC   count_bergs                  ! routine called in icbdia, icbini, icblbc, icbrst modules
   PUBLIC   increment_kounter            ! routine called in icbini, icbclv modules
   PUBLIC   yearday                      ! routine called in icbclv, icbrun module
   PUBLIC   sum_mass                     ! routine called in icbdia module
   PUBLIC   sum_heat                     ! routine called in icbdia module

   PRIVATE  create_iceberg

   !!-------------------------------------------------------------------------

CONTAINS

   SUBROUTINE copy_flds()
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE copy_flds  ***
      !!
      !! ** Purpose :   iceberg initialization.
      !!
      !! ** Method  : - blah blah
      !!----------------------------------------------------------------------

      ! copy nemo forcing arrays into iceberg versions with extra halo
      ! only necessary for variables not on T points
      ! and ssh which is used to calculate gradients

      uo_e(:,:) = 0._wp ;   uo_e(1:jpi, 1:jpj) = ssu_m(:,:)
      vo_e(:,:) = 0._wp ;   vo_e(1:jpi, 1:jpj) = ssv_m(:,:)
      ff_e(:,:) = 0._wp ;   ff_e(1:jpi, 1:jpj) = ff   (:,:)
      ua_e(:,:) = 0._wp ;   ua_e(1:jpi, 1:jpj) = utau (:,:)
      va_e(:,:) = 0._wp ;   va_e(1:jpi, 1:jpj) = vtau (:,:)

      CALL lbc_lnk_e( uo_e, 'U', -1._wp, 1, 1 )
      CALL lbc_lnk_e( vo_e, 'V', -1._wp, 1, 1 )
      CALL lbc_lnk_e( ff_e, 'F', +1._wp, 1, 1 )
      CALL lbc_lnk_e( ua_e, 'U', -1._wp, 1, 1 )
      CALL lbc_lnk_e( va_e, 'V', -1._wp, 1, 1 )

#if defined key_lim2 || defined key_lim3
      ui_e(:,:) = 0._wp ;   ui_e(1:jpi, 1:jpj) = u_ice(:,:)
      vi_e(:,:) = 0._wp ;   vi_e(1:jpi, 1:jpj) = v_ice(:,:)

      CALL lbc_lnk_e( ui_e, 'U', -1._wp, 1, 1 )
      CALL lbc_lnk_e( vi_e, 'V', -1._wp, 1, 1 )
#endif

      !! special for ssh which is used to calculate slope
      !! so fudge some numbers all the way around the boundary

      ssh_e(:,:) = 0._wp ;   ssh_e(1:jpi, 1:jpj) = ssh_m(:,:)
      ssh_e(0    ,    :) = ssh_e(1  ,  :)
      ssh_e(jpi+1,    :) = ssh_e(jpi,  :)
      ssh_e(:    ,    0) = ssh_e(:  ,  1)
      ssh_e(:    ,jpj+1) = ssh_e(:  ,jpj)
      ssh_e(0,0)         = ssh_e(1,1)
      ssh_e(jpi+1,0)     = ssh_e(jpi,1)
      ssh_e(0,jpj+1)     = ssh_e(1,jpj)
      ssh_e(jpi+1,jpj+1) = ssh_e(jpi,jpj)
      CALL lbc_lnk_e( ssh_e, 'T', +1._wp, 1, 1 )

   END SUBROUTINE copy_flds

   !!-------------------------------------------------------------------------

   SUBROUTINE interp_flds( pi, pe1, puo, pui, pua, pssh_i,   &
      &                    pj, pe2, pvo, pvi, pva, pssh_j,   &
      &                    psst, pcn, phi, pff )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE interp_flds  ***
      !!
      !! ** Purpose :   iceberg initialization.
      !!
      !! ** Method  : - blah blah
      !!
      !!       !!gm  CAUTION here I do not care of the slip/no-slip conditions
      !!             this can be done later (not that easy to do...)
      !!             right now, U is 0 in land so that the coastal value of velocity parallel to the coast
      !!             is half the off shore value, wile the normal-to-the-coast value is zero.
      !!             This is OK as a starting point.
      !!
      !!----------------------------------------------------------------------
      REAL(wp), INTENT(in   ) ::   pi , pj                        ! position in (i,j) referential
      REAL(wp), INTENT(  out) ::   pe1, pe2                       ! i- and j scale factors
      REAL(wp), INTENT(  out) ::   puo, pvo, pui, pvi, pua, pva   ! ocean, ice and wind speeds
      REAL(wp), INTENT(  out) ::   pssh_i, pssh_j                 ! ssh i- & j-gradients
      REAL(wp), INTENT(  out) ::   psst, pcn, phi, pff            ! SST, ice concentration, ice thickness, Coriolis
      !
      REAL(wp) ::   zcd, zmod       ! local scalars
      !!----------------------------------------------------------------------

      pe1 = bilin_e( e1t, e1u, e1v, e1f, pi, pj )                 ! scale factors
      pe2 = bilin_e( e2t, e2u, e2v, e2f, pi, pj )
      !
      puo = bilin( uo_e, pi, pj, 'U', 1, 1 )                      ! ocean velocities
      pvo = bilin( vo_e, pi, pj, 'V', 1, 1 )
      psst = bilin( sst_m, pi, pj, 'T', 0, 0 )                    ! SST
      pcn  = bilin( fr_i , pi, pj, 'T', 0, 0 )                    ! ice concentration
      pff  = bilin( ff_e , pi, pj, 'F', 1, 1 )                    ! Coriolis parameter
      !
      pua = bilin( ua_e , pi, pj, 'U', 1, 1 )                     ! 10m wind
      pva = bilin( va_e , pi, pj, 'V', 1, 1 )                     ! here (ua,va) are stress => rough conversion from stress to speed
      zcd  = 1.22_wp * 1.5e-3_wp                                  ! air density * drag coefficient
      zmod = 1._wp / MAX(  1.e-20, SQRT(  zcd * SQRT( pua*pua + pva*pva)  )  )
      pua = pua * zmod                                            ! note: stress module=0 necessarly implies ua=va=0
      pva = pva * zmod

#if defined key_lim2 || defined key_lim3
      pui = bilin( ui_e, pi, pj, 'U', 1, 1 )                      ! sea-ice velocities
      pvi = bilin( vi_e, pi, pj, 'V', 1, 1 )
      phi = bilin( hi  , pi, pj, 'T', 0, 0 )                      ! ice thickness
#else
      pui = 0._wp
      pvi = 0._wp
      phi = 0._wp
#endif

      ! Estimate SSH gradient in i- and j-direction (centred evaluation)
      pssh_i = ( bilin( ssh_e, pi+0.1_wp, pj, 'T', 1, 1 ) - bilin( ssh_e, pi-0.1_wp, pj, 'T', 1, 1 )  ) / ( 0.2_wp * pe1 )
      pssh_j = ( bilin( ssh_e, pi, pj+0.1_wp, 'T', 1, 1 ) - bilin( ssh_e, pi, pj-0.1_wp, 'T', 1, 1 )  ) / ( 0.2_wp * pe2 )
      !
   END SUBROUTINE interp_flds

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION bilin( pfld, pi, pj, cd_type, kdi, kdj )
      !!----------------------------------------------------------------------
      !!                  ***  FUNCTION bilin  ***
      !!
      !! ** Purpose :   bilinear interpolation at berg location depending on the grid-point type
      !!
      !!       !!gm  CAUTION an optional argument should be added to handle
      !!             the slip/no-slip conditions  ==>>> to be done later
      !!
      !!----------------------------------------------------------------------
      INTEGER                                         , INTENT(in) ::   kdi, kdj  ! extra halo on grid
      REAL(wp), DIMENSION(1-kdi:jpi+kdi,1-kdj:jpj+kdj), INTENT(in) ::   pfld      ! field to be interpolated
      REAL(wp)                                        , INTENT(in) ::   pi, pj    ! targeted coordinates in (i,j) referential
      CHARACTER(len=1)                                , INTENT(in) ::   cd_type   ! type of pfld array grid-points: = T , U , V or F points
      !
      INTEGER  ::   ii, ij   ! local integer
      REAL(wp) ::   zi, zj   ! local real
      !!----------------------------------------------------------------------
      !
      SELECT CASE ( cd_type )
         CASE ( 'T' )
            ! note that here there is no +0.5 added
            ! since we're looking for four T points containing quadrant we're in of 
            ! current T cell
            ii = INT( pi     )
            ij = INT( pj      )    ! T-point
            zi = pi - REAL(ii,wp)
            zj = pj - REAL(ij,wp)
         CASE ( 'U' )
            ii = INT( pi-0.5 )
            ij = INT( pj      )    ! U-point
            zi = pi - 0.5 - REAL(ii,wp)
            zj = pj - REAL(ij,wp)
         CASE ( 'V' )
            ii = INT( pi     )
            ij = INT( pj -0.5 )    ! V-point
            zi = pi - REAL(ii,wp)
            zj = pj - 0.5 - REAL(ij,wp)
         CASE ( 'F' )
            ii = INT( pi-0.5 )
            ij = INT( pj -0.5 )    ! F-point
            zi = pi - 0.5 - REAL(ii,wp)
            zj = pj - 0.5 - REAL(ij,wp)
      END SELECT
      !
      ! find position in this processor
      ii = ii - nimpp + 1
      ij = ij - njmpp + 1
      !
      bilin = ( pfld(ii,ij  ) * (1.-zi) + pfld(ii+1,ij  ) * zi ) * (1.-zj)   &
         &  + ( pfld(ii,ij+1) * (1.-zi) + pfld(ii+1,ij+1) * zi ) *     zj
      !
   END FUNCTION bilin

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION bilin_x( pfld, pi, pj )
      !!----------------------------------------------------------------------
      !!                  ***  FUNCTION bilin_x  ***
      !!
      !! ** Purpose :   bilinear interpolation at berg location depending on the grid-point type
      !!                Special case for interpolating longitude
      !!
      !!       !!gm  CAUTION an optional argument should be added to handle
      !!             the slip/no-slip conditions  ==>>> to be done later
      !!
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in) ::   pfld      ! field to be interpolated
      REAL(wp)                    , INTENT(in) ::   pi, pj    ! targeted coordinates in (i,j) referential
      !
      INTEGER                                  ::   ii, ij   ! local integer
      REAL(wp)                                 ::   zi, zj   ! local real
      REAL(wp), DIMENSION(4)                   ::   z4
      !!----------------------------------------------------------------------
      !
      ! note that here there is no +0.5 added
      ! since we're looking for four T points containing quadrant we're in of 
      ! current T cell
      ii = INT( pi     )
      ij = INT( pj      )    ! T-point
      zi = pi - REAL(ii,wp)
      zj = pj - REAL(ij,wp)
      !
      ! find position in this processor
      ii = ii - nimpp + 1
      ij = ij - njmpp + 1
      z4(1) = pfld(ii  ,ij  )
      z4(2) = pfld(ii+1,ij  )
      z4(3) = pfld(ii  ,ij+1)
      z4(4) = pfld(ii+1,ij+1)
      IF( MAXVAL(z4) - MINVAL(z4) > 90._wp ) THEN
         WHERE( z4 < 0._wp ) z4 = z4 + 360._wp
      ENDIF
      !
      bilin_x = (z4(1) * (1.-zi) + z4(2) * zi) * (1.-zj) + (z4(3) * (1.-zi) + z4(4) * zi) * zj
      IF( bilin_x > 180._wp ) bilin_x = bilin_x - 360._wp
      !
   END FUNCTION bilin_x

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION bilin_e( pet, peu, pev, pef, pi, pj )
      !!----------------------------------------------------------------------
      !!                  ***  FUNCTION dom_init  ***
      !!
      !! ** Purpose :   bilinear interpolation at berg location of horizontal scale factor
      !! ** Method  :   interpolation done using the 4 nearest grid point among
      !!                t-, u-, v-, and f-points.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:,:), INTENT(in) ::   pet, peu, pev, pef   ! horizontal scale factor to be interpolated at t-,u-,v- & f-pts
      REAL(wp)                , INTENT(in) ::   pi, pj               ! targeted coordinates in (i,j) referential
      !
      INTEGER  ::   ii, ij, icase   ! local integer
      !
      ! weights corresponding to corner points of a T cell quadrant
      REAL(wp) ::   zi, zj          ! local real
      !
      ! values at corner points of a T cell quadrant
      ! 00 = bottom left, 10 = bottom right, 01 = top left, 11 = top right
      REAL(wp) ::   ze00, ze10, ze01, ze11
      !!----------------------------------------------------------------------
      !
      ii = INT( pi )   ;   ij = INT( pj )    ! left bottom T-point (i,j) indices

      ! fractional box spacing
      ! 0   <= zi < 0.5, 0   <= zj < 0.5   -->  NW quadrant of current T cell
      ! 0.5 <= zi < 1  , 0   <= zj < 0.5   -->  NE quadrant
      ! 0   <= zi < 0.5, 0.5 <= zj < 1     -->  SE quadrant
      ! 0.5 <= zi < 1  , 0.5 <= zj < 1     -->  SW quadrant

      zi = pi - REAL(ii,wp)
      zj = pj - REAL(ij,wp)

      ! find position in this processor
      ii = ii - nimpp + 1
      ij = ij - njmpp + 1

      IF(    0.0_wp <= zi .AND. zi < 0.5_wp   ) THEN
         IF( 0.0_wp <= zj .AND. zj < 0.5_wp        )   THEN        !  NE quadrant
            !                                                      !             i=I       i=I+1/2
            ze01 = pev(ii  ,ij  )   ;   ze11 = pef(ii  ,ij  )      !   j=J+1/2    V ------- F
            ze00 = pet(ii  ,ij  )   ;   ze10 = peu(ii  ,ij  )      !   j=J        T ------- U
            zi = 2._wp * zi
            zj = 2._wp * zj
         ELSE                                                      !  SE quadrant
            !                                                                    !             i=I       i=I+1/2
            ze01 = pet(ii  ,ij+1)   ;   ze11 = peu(ii  ,ij+1)      !   j=J+1      T ------- U
            ze00 = pev(ii  ,ij  )   ;   ze10 = pef(ii  ,ij  )      !   j=J+1/2    V ------- F
            zi = 2._wp *  zi
            zj = 2._wp * (zj-0.5_wp)
         ENDIF
      ELSE
         IF( 0.0_wp <= zj .AND. zj < 0.5_wp        )   THEN        !  NW quadrant
            !                                                                    !             i=I       i=I+1/2
            ze01 = pef(ii  ,ij  )   ;   ze11 = pev(ii+1,ij)        !   j=J+1/2    F ------- V
            ze00 = peu(ii  ,ij  )   ;   ze10 = pet(ii+1,ij)        !   j=J        U ------- T
            zi = 2._wp * (zi-0.5_wp)
            zj = 2._wp *  zj
         ELSE                                                      !  SW quadrant
            !                                                                    !             i=I+1/2   i=I+1
            ze01 = peu(ii  ,ij+1)   ;   ze11 = pet(ii+1,ij+1)      !   j=J+1      U ------- T
            ze00 = pef(ii  ,ij  )   ;   ze10 = pev(ii+1,ij  )      !   j=J+1/2    F ------- V
            zi = 2._wp * (zi-0.5_wp)
            zj = 2._wp * (zj-0.5_wp)
         ENDIF
      ENDIF
      !
      bilin_e = ( ze01 * (1.-zi) + ze11 * zi ) *     zj    &
         &    + ( ze00 * (1.-zi) + ze10 * zi ) * (1.-zj)
      !
   END FUNCTION bilin_e

   !!-------------------------------------------------------------------------


   SUBROUTINE add_new_berg_to_list( bergvals, ptvals )
      !!----------------------------------------------------------------------
      !!                ***  ROUTINE add_new_berg_to_list  ***
      !!
      !! ** Purpose :   add a new berg to the iceberg list
      !!
      !! ** method  : - ?
      !!----------------------------------------------------------------------
      TYPE(iceberg), INTENT(in)           ::   bergvals
      TYPE(point)  , INTENT(in)           ::   ptvals
      !
      TYPE(iceberg), POINTER ::   new => NULL()
      !!----------------------------------------------------------------------
      !
      new => NULL()
      CALL create_iceberg( new, bergvals, ptvals )
      CALL insert_berg_into_list( new )
      new => NULL()     ! Clear new
      !
   END SUBROUTINE add_new_berg_to_list

   !!-------------------------------------------------------------------------

   SUBROUTINE create_iceberg(berg, bergvals, ptvals )
      ! Arguments
      TYPE(iceberg), INTENT(in) :: bergvals
      TYPE(point)  , INTENT(in) :: ptvals
      TYPE(iceberg), POINTER    :: berg
      ! local variables
      TYPE(point)  , POINTER    :: pt
      INTEGER                   :: istat

      IF ( ASSOCIATED(berg) ) then
         CALL ctl_stop( 'icebergs, create_iceberg: berg already associated' )
      ENDIF
      ALLOCATE(berg, STAT=istat)
      IF( istat /= 0 ) CALL ctl_stop( 'failed to allocate iceberg' )
      berg%number(:) = bergvals%number(:)
      berg%mass_scaling = bergvals%mass_scaling
      berg%prev => NULL()
      berg%next => NULL()

      ALLOCATE(pt, STAT=istat)
      IF( istat /= 0 ) CALL ctl_stop( 'failed to allocate first iceberg point' )
      pt = ptvals
      berg%current_point => pt

   END SUBROUTINE create_iceberg

   !!-------------------------------------------------------------------------

   SUBROUTINE insert_berg_into_list( newberg )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE insert_berg_into_list  ***
      !!
      !! ** Purpose :   add a new berg to the iceberg list
      !!
      !! ** method  : - ?
      !!----------------------------------------------------------------------
      TYPE(iceberg), POINTER  ::   newberg
      !
      TYPE(iceberg), POINTER  ::   this, prev, last
      !!----------------------------------------------------------------------

      IF( ASSOCIATED( first_berg ) ) THEN
!        last = last_berg()
         last=>first_berg
         DO WHILE (ASSOCIATED(last%next))
            last=>last%next
         ENDDO
         newberg%prev => last
         last%next    => newberg
         last         => newberg
      ELSE                       ! list is empty so create it
         first_berg => newberg
      ENDIF
      !
   END SUBROUTINE insert_berg_into_list

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION yearday(kmon, kday, khr, kmin, ksec)
      ! sga - improved but still only applies to 365 day year, need to do this properly
      ! Arguments
      INTEGER, intent(in)     :: kmon, kday, khr, kmin, ksec
      ! Local variables
      INTEGER, DIMENSION(12)  :: imonths = (/ 0,31,28,31,30,31,30,31,31,30,31,30 /)

      yearday = FLOAT( SUM( imonths(1:kmon) ) )
      yearday = yearday + FLOAT(kday-1) + (FLOAT(khr) + (FLOAT(kmin) + FLOAT(ksec)/60.)/60.)/24.

   END FUNCTION yearday

   !!-------------------------------------------------------------------------

   SUBROUTINE delete_iceberg_from_list( first, berg )
      ! Arguments
      TYPE(iceberg), POINTER :: first, berg

      ! Connect neighbors to each other
      IF ( ASSOCIATED(berg%prev) ) THEN
        berg%prev%next => berg%next
      ELSE
        first => berg%next
      ENDIF
      IF (ASSOCIATED(berg%next)) berg%next%prev => berg%prev

      ! Bye-bye berg
      CALL destroy_iceberg(berg)

   END SUBROUTINE delete_iceberg_from_list

   !!-------------------------------------------------------------------------

   SUBROUTINE destroy_iceberg( berg )
      ! Arguments
      TYPE(iceberg), POINTER :: berg

      ! Remove any points
      IF( ASSOCIATED( berg%current_point ) ) DEALLOCATE( berg%current_point )

      ! Bye-bye berg
      DEALLOCATE(berg)

   END SUBROUTINE destroy_iceberg

   !!-------------------------------------------------------------------------

   SUBROUTINE track_berg( knum, cd_label, kt )
      ! Arguments
      INTEGER, DIMENSION(nkounts)    :: knum            ! iceberg number
      CHARACTER(len=*)               :: cd_label
      INTEGER                        :: kt             ! timestep number
      ! Local variables
      TYPE(iceberg), POINTER         :: this
      LOGICAL                        :: match
      INTEGER                        :: k

      this => first_berg
      DO WHILE( ASSOCIATED(this) )
         match = .TRUE.
         DO k=1,nkounts
            IF( this%number(k) /= knum(k) ) match = .FALSE.
         END DO
         IF( match ) CALL print_berg(this, kt)
         this => this%next
      ENDDO

   END SUBROUTINE track_berg

   !!-------------------------------------------------------------------------

   SUBROUTINE print_berg( berg, kt )
      ! Arguments
      TYPE(iceberg), POINTER :: berg
      TYPE(point)  , POINTER :: pt
      INTEGER                :: kt             ! timestep number

      pt => berg%current_point
      WRITE(numicb, 9200) kt, berg%number(1), &
                   pt%xi, pt%yj, pt%lon, pt%lat, pt%uvel, pt%vvel,  &
                   pt%uo, pt%vo, pt%ua, pt%va, pt%ui, pt%vi
      CALL flush( numicb )
 9200 FORMAT(5x,i5,2x,i10,6(2x,2f10.4))

   END SUBROUTINE print_berg

   !!-------------------------------------------------------------------------

   SUBROUTINE print_bergs( cd_label, kt )
      ! Arguments
      CHARACTER(len=*)       :: cd_label
      INTEGER                :: kt             ! timestep number
      ! Local variables
      INTEGER                :: ibergs, inbergs
      TYPE(iceberg), POINTER :: this

      this => first_berg
      IF( ASSOCIATED(this) ) THEN
         WRITE(numicb,'(a," pe=(",i3,")")' ) cd_label, narea
         WRITE(numicb,'(a8,4x,a6,12x,a5,15x,a7,19x,a3,17x,a5,17x,a5,17x,a5)' )   &
                      'timestep', 'number', 'xi,yj','lon,lat','u,v','uo,vo','ua,va','ui,vi'
      ENDIF
      DO WHILE( ASSOCIATED(this) )
        CALL print_berg(this, kt)
        this => this%next
      ENDDO
      ibergs = count_bergs()
      inbergs = ibergs
      IF( lk_mpp ) CALL mpp_sum(inbergs)
      IF ( ibergs .GT. 0 ) WRITE(numicb,'(a," there are",i5," bergs out of",i6," on PE ",i4)') &
                                           cd_label, ibergs, inbergs, narea

   END SUBROUTINE print_bergs

   !!-------------------------------------------------------------------------

   SUBROUTINE increment_kounter()
      ! Small routine for coping with very large integer values labelling icebergs
      ! num_bergs is a array of integers
      ! the first member is incremented in steps of jpnij starting from narea
      ! this means each iceberg is labelled with a unique number
      ! when this gets to the maximum allowed integer the second and subsequent members are 
      ! used to count how many times the member before cycles

      ! Local variables
      INTEGER                :: ii, ibig

      ibig = HUGE(num_bergs(1))
      IF( ibig-jpnij < num_bergs(1) ) THEN
         num_bergs(1) = narea
         DO ii = 2,nkounts
            IF( num_bergs(ii) == ibig ) THEN
               num_bergs(ii) = 0
               IF( ii == nkounts ) CALL ctl_stop('Sorry, run out of iceberg number space')
            ELSE
               num_bergs(ii) = num_bergs(ii) + 1
               EXIT
            ENDIF
         END DO
      ELSE
         num_bergs(1) = num_bergs(1) + jpnij
      ENDIF

   END SUBROUTINE increment_kounter

   !!-------------------------------------------------------------------------

   INTEGER function count_bergs()
      ! Local variables
      TYPE(iceberg), POINTER :: this

      count_bergs = 0
      this => first_berg
      DO WHILE( ASSOCIATED(this) )
         count_bergs = count_bergs+1
         this => this%next
      ENDDO

   END FUNCTION count_bergs

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION sum_mass( first, justbits, justbergs )
      !!----------------------------------------------------------------------
      !!                 ***  FUNCTION sum_mass  ***
      !!
      !! ** Purpose :   compute the mass all iceberg, all bergies or all bergs.
      !!----------------------------------------------------------------------
      TYPE(iceberg)      , POINTER  ::   first
      TYPE(point)        , POINTER  ::   pt
      LOGICAL, INTENT(in), OPTIONAL ::   justbits, justbergs
      !
      TYPE(iceberg), POINTER ::   this
      !!----------------------------------------------------------------------
      sum_mass = 0._wp
      this => first
      !
      IF( PRESENT( justbergs  ) ) THEN
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_mass = sum_mass + pt%mass         * this%mass_scaling
            this => this%next
         END DO
      ELSEIF( PRESENT(justbits) ) THEN
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_mass = sum_mass + pt%mass_of_bits * this%mass_scaling
            this => this%next
         END DO
      ELSE
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_mass = sum_mass + ( pt%mass + pt%mass_of_bits ) * this%mass_scaling
            this => this%next
         END DO
      ENDIF
      !
   END FUNCTION sum_mass

   !!-------------------------------------------------------------------------

   REAL(wp) FUNCTION sum_heat( first, justbits, justbergs )
      !!----------------------------------------------------------------------
      !!                 ***  FUNCTION sum_heat  ***
      !!
      !! ** Purpose :   compute the heat in all iceberg, all bergies or all bergs.
      !!----------------------------------------------------------------------
      TYPE(iceberg)      , POINTER  ::   first
      LOGICAL, INTENT(in), OPTIONAL ::   justbits, justbergs
      !
      TYPE(iceberg)      , POINTER  ::   this
      TYPE(point)        , POINTER  ::   pt
      !!----------------------------------------------------------------------
      sum_heat = 0._wp
      this => first
      !
      IF( PRESENT( justbergs  ) ) THEN
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_heat = sum_heat + pt%mass         * this%mass_scaling * pt%heat_density
            this => this%next
         END DO
      ELSEIF( PRESENT(justbits) ) THEN
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_heat = sum_heat + pt%mass_of_bits * this%mass_scaling * pt%heat_density
            this => this%next
         END DO
      ELSE
         DO WHILE( ASSOCIATED( this ) )
            pt => this%current_point
            sum_heat = sum_heat + ( pt%mass + pt%mass_of_bits ) * this%mass_scaling * pt%heat_density
            this => this%next
         END DO
      ENDIF
      !
   END FUNCTION sum_heat

   !!-------------------------------------------------------------------------

END MODULE icbutl
